#!/usr/bin/env python
import sys
from coverage import coverage
from optparse import OptionParser

from django.conf import settings

if not settings.configured:
    extra_settings = {
        'PAYPAL_EXPRESS_URL': 'https://www.sandbox.paypal.com/webscr',
        'PAYPAL_SANDBOX_MODE': True,
        'PAYPAL_VERSION': '88.0',
        'PAYPAL_PAYFLOW_TEST_MODE': True,
    }
    # To specify integration settings (which include passwords, hence why they
    # are not committed), create an integration.py module.
    try:
        from integration import *
    except ImportError:
        extra_settings.update({
            'PAYPAL_API_USERNAME': '',
            'PAYPAL_API_PASSWORD': '',
            'PAYPAL_API_SIGNATURE': '',
            'PAYPAL_PAYFLOW_VENDOR_ID': '',
            'PAYPAL_PAYFLOW_PASSWORD': '',
        })
    else:
        for key, value in list(locals().items()):
            if key.startswith('PAYPAL'):
                extra_settings[key] = value

    from oscar.defaults import *
    for key, value in list(locals().items()):
        if key.startswith('OSCAR'):
            extra_settings[key] = value
    extra_settings['OSCAR_ALLOW_ANON_CHECKOUT'] = True

    from oscar import get_core_apps, OSCAR_MAIN_TEMPLATE_DIR

    settings.configure(
        DATABASES={
            'default': {
                'ENGINE': 'django.db.backends.sqlite3',
            }
        },
        INSTALLED_APPS=[
            'django.contrib.auth',
            'django.contrib.admin',
            'django.contrib.contenttypes',
            'django.contrib.sessions',
            'django.contrib.sites',
            'django.contrib.flatpages',
            'django.contrib.staticfiles',
            'paypal',
            'compressor',
        ] + get_core_apps([
            'tests.shipping',
        ]),
        MIDDLEWARE_CLASSES=(
            'django.middleware.common.CommonMiddleware',
            'django.contrib.sessions.middleware.SessionMiddleware',
            'django.middleware.csrf.CsrfViewMiddleware',
            'django.contrib.auth.middleware.AuthenticationMiddleware',
            'django.contrib.messages.middleware.MessageMiddleware',
            'oscar.apps.basket.middleware.BasketMiddleware',
        ),
        TEMPLATE_CONTEXT_PROCESSORS = (
            "django.contrib.auth.context_processors.auth",
            "django.core.context_processors.request",
            "django.core.context_processors.debug",
            "django.core.context_processors.i18n",
            "django.core.context_processors.media",
            "django.core.context_processors.static",
            "django.contrib.messages.context_processors.messages",
            # Oscar specific
            'oscar.apps.search.context_processors.search_form',
            'oscar.apps.promotions.context_processors.promotions',
            'oscar.apps.checkout.context_processors.checkout',
            'oscar.core.context_processors.metadata',
            'oscar.apps.customer.notifications.context_processors.notifications',
        ),
        DEBUG=False,
        HAYSTACK_CONNECTIONS={
            'default': {
                'ENGINE': 'haystack.backends.simple_backend.SimpleEngine',
            },
        },
        TEMPLATE_DIRS=(OSCAR_MAIN_TEMPLATE_DIR,),
        SITE_ID=1,
        ROOT_URLCONF='tests.urls',
        COMPRESS_ENABLED=False,
        STATIC_URL='/',
        STATIC_ROOT='/static/',
        NOSE_ARGS=['-s', '--with-spec'],
        # Oscar 1.0 factories assume this setting is present. Fixed in 1.1.
        OSCAR_INITIAL_ORDER_STATUS='A',
        **extra_settings
    )

from django_nose import NoseTestSuiteRunner


def run_tests(*test_args):
    if not test_args:
        test_args = ['tests']

    # Run tests
    test_runner = NoseTestSuiteRunner(verbosity=1)

    c = coverage(source=['paypal'], omit=['*migrations*', '*tests*'],
                 auto_data=True)
    c.start()
    num_failures = test_runner.run_tests(test_args)
    c.stop()

    if num_failures > 0:
        sys.exit(num_failures)
    print("Generating HTML coverage report")
    c.html_report()


if __name__ == '__main__':
    parser = OptionParser()
    (options, args) = parser.parse_args()
    run_tests(*args)



from django.utils.translation import ugettext_lazy as _


class MissingShippingMethodException(Exception):

    def __init__(self, value):
        self.message = _(u"Shipping method expected, received: {}".format(value))


class MissingShippingAddressException(Exception):

    def __init__(self, value):
        self.message = _(u"Shipping address expected, received: {}".format(value))


class TransactionException(Exception):

    def __init__(self, *args, **kwargs):
        super(TransactionException, *args, **kwargs)


class AuthenticationError(TransactionException):
    pass


class FailedTransaction(TransactionException):

    def __init__(self, cause=None, *args, **kwargs):
        self.message = _(u'Failed transaction motive: {}'.format(cause if cause else _(u'Unknown')))
        super(FailedTransaction, self).__init__(*args, **kwargs)


class TimeLimitExceeded(TransactionException):
    pass


class AbortedTransactionByCardHolder(TransactionException):
    pass


class U3Exception(TransactionException):
    """ Internal error. """
    pass



XXXXXXXXXX
XXXXXXXXXX
XXXXXXXXXXX
XXXXXXXXX
XXXXXXXXXXXXXXXX
XXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX


"""
@author     Allware Ltda. (http://www.allware.cl)
@copyright  2016 Transbank S.A. (http://www.tranbank.cl)
@date       Jan 2015
@license    GNU LGPL
@version    2.0.1
"""
from django.conf import settings
from django.utils.translation import ugettext_lazy as _
from django.http import HttpResponse
import socket

from oscar_webpay.certificates import cert_normal

from suds.client import Client
from suds.wsse import Security, Timestamp
from wsse.suds import WssePlugin

from suds.transport.https import HttpTransport
import logging

from decimal import *

from oscar_webpay.oscar_webpay_settings import OscarWebpaySettingError

logging.basicConfig()

config = None
url = None


class Dictionaries():
    """
		Configuracion de WSDL segun ambiente
		"""

    @staticmethod
    def dictionaryConfig():
        normal_config = getattr(settings, 'WEBPAY_ONECLICK', None)
        config = normal_config.get('ENVIRONMENTS', None)
        if config is None:
            raise OscarWebpaySettingError(_('You must provide the ENVIRONMENTS variable for the settings'))

        return config


class WebpayOneClick():
    def __init__(self, configuration):

        global config
        config = configuration
        self.__config = config

        dictWsdl = Dictionaries.dictionaryConfig()

        global url
        url = dictWsdl[self.__config.getEnvironment()]
        self.__url = url

    """
		Permite realizar la inscripcion del tarjetahabiente e informacion de su 
		tarjeta de crdito. Retorna como respuesta un token que representa la transaccion de inscripcion
		y una URL (UrlWebpay),que corresponde a la URL de inscripcion de One Click
		"""

    @staticmethod
    def initInscription(username, email, urlReturn):

        client = WebpayOneClick.get_client(url, config.getPrivateKey(), config.getPublicCert(), config.getWebPayCert())
        client.options.cache.clear()

        oneClickInscriptionInput = client.factory.create('oneClickInscriptionInput')

        """ nombre de usuario """
        oneClickInscriptionInput.username = username

        """ email """
        oneClickInscriptionInput.email = email

        """ url de respuesta """
        oneClickInscriptionInput.responseURL = urlReturn

        try:
            oneClickInscriptionOutput = client.service.initInscription(oneClickInscriptionInput)
        except Exception as e:
            print str(e)

        return oneClickInscriptionOutput

    """
		Permite finalizar el proceso de inscripcion del tarjetahabiente en Oneclick. Entre otras cosas, retorna el identificador del usuario en Oneclick,
		el cual sera utilizado para realizar las transacciones de pago
		"""

    @staticmethod
    def finishInscription(token):

        client = WebpayOneClick.get_client(url, config.getPrivateKey(), config.getPublicCert(), config.getWebPayCert())

        client.options.cache.clear()

        oneClickFinishInscriptionInput = client.factory.create('oneClickFinishInscriptionInput')

        """token obtenido en initInscription """
        oneClickFinishInscriptionInput.token = token

        try:
            oneClickFinishInscriptionOutput = client.service.finishInscription(oneClickFinishInscriptionInput)
        except Exception as e:
            print str(e)

        return oneClickFinishInscriptionOutput

    """
		Permite realizar transacciones de pago. Retorna el resultado de la autorizacion. Este metodo que debe ser ejecutado, cada vez que el usuario
		selecciona pagar con Oneclick
        """

    @staticmethod
    def authorize(buyOrder, tbkUser, username, amount):

        client = WebpayOneClick.get_client(url, config.getPrivateKey(), config.getPublicCert(), config.getWebPayCert())

        client.options.cache.clear()

        oneClickPayInput = client.factory.create('oneClickPayInput')

        """ Orden de Compra """
        oneClickPayInput.buyOrder = buyOrder

        """ Usuario TRansbank """
        oneClickPayInput.tbkUser = tbkUser

        """ Nombre de usuario """
        oneClickPayInput.username = username

        """ Monto de Compra"""
        oneClickPayInput.amount = amount

        try:
            oneClickPayOutput = client.service.authorize(oneClickPayInput)
        except Exception as e:
            print str(e)

        return oneClickPayOutput

    """
		Permite reversar una transaccion de venta autorizada con anterioridad. Este metodo retorna como respuesta un identificador unico de la transaccion de reversa.
		"""

    @staticmethod
    def reverseTransaction(buyOrder):

        client = WebpayOneClick.get_client(url, config.getPrivateKey(), config.getPublicCert(), config.getWebPayCert())

        client.options.cache.clear()

        oneClickReverseInput = client.factory.create('oneClickReverseInput')

        oneClickReverseInput.buyorder = buyOrder

        try:
            oneClickReverseOutput = client.service.codeReverseOneClick(oneClickReverseInput)
        except Exception as e:
            print str(e)

        return oneClickReverseOutput

    """
		Permite eliminar la inscripcion de un usuario en Webpay OneClick ya sea por la eliminacion de un cliente en su sistema o por la solicitud de este para no operar con esta forma de pago.
		"""

    @staticmethod
    def removeUser(tbkUser, username):

        client = WebpayOneClick.get_client(url, config.getPrivateKey(), config.getPublicCert(), config.getWebPayCert())

        client.options.cache.clear()

        oneClickRemoveUserInput = client.factory.create('oneClickRemoveUserInput')

        """ Usuario Transbank """
        oneClickRemoveUserInput.tbkUser = tbkUser

        """ Nombre de Usuario """
        oneClickRemoveUserInput.username = username

        try:
            return client.service.removeUser(oneClickRemoveUserInput)
        except Exception as e:
            print str(e)

    @staticmethod
    def get_client(wsdl_url, our_keyfile_path, our_certfile_path, their_certfile_path):

        transport = HttpTransport()
        wsse = Security()

        return Client(
            wsdl_url,
            transport=transport,
            wsse=wsse,
            plugins=[
                WssePlugin(
                    keyfile=our_keyfile_path,
                    certfile=our_certfile_path,
                    their_certfile=their_certfile_path,
                ),
            ],
        )



from configuration import Configuration
from webpay_normal import WebpayNormal
from webpay_mall_normal import WebpayMallNormal
"""
from webpay_nullify import WebpayNullify
"""
from webpay_complete import WebpayComplete
"""
from webpay_capture import WebpayCapture
"""
from webpay_oneclick import WebpayOneClick


class Webpay:
    
        def __init__(self, params):
            self.__configuration = params
            return None

        def getNormalTransaction(self):
            webpayNormal = WebpayNormal(self.__configuration)
            return webpayNormal

        def getMallNormalTransaction(self):
            webpayMallNormal = WebpayMallNormal(self.__configuration)
            return webpayMallNormal
        
        """
        def getNullifyTransaction(self):
            webpayNullify = WebpayNullify(self.__configuration)
            return webpayNullify
        """
        
        def getCompleteTransaction(self):
            webpayComplete = WebpayComplete(self.__configuration)
            return webpayComplete
        
        """
        def getCaptureTransaction(self):
            webpayCapture = WebpayCapture(self.__configuration)
            return webpayCapture
        """
        
        def getOneClickTransaction(self):
            webpayOneClick = WebpayOneClick(self.__configuration)
            return webpayOneClick
        


"""
@author     Allware Ltda. (http://www.allware.cl)
@copyright  2016 Transbank S.A. (http://www.tranbank.cl)
@date       Jan 2015
@license    GNU LGPL
@version    2.0.1
"""
from django.conf import settings
from django.http import HttpResponse
import socket

from certificates import cert_capture

from suds.client import Client
from suds.wsse import Security, Timestamp
from wsse.suds import WssePlugin

from suds.transport.https import HttpTransport
import logging

from decimal import *

from oscar_webpay.oscar_webpay_settings import OscarWebpaySettingError
from django.utils.translation import ugettext_lazy as _

logging.basicConfig()

config = None
url = None


class Dictionaries():
    """
	Configuracion de WSDL segun ambiente
	"""

    @staticmethod
    def dictionaryConfig():
        normal_config = getattr(settings, 'WEBPAY_CAPTURE', None)
        config = normal_config.get('ENVIRONMENTS', None)
        if config is None:
            raise OscarWebpaySettingError(_('You must provide the ENVIRONMENTS variable for the settings'))

        return config


class WebpayCapture():
    def __init__(self, configuration):

        global config
        config = configuration
        self.__config = config

        dictWsdl = Dictionaries.dictionaryConfig()

        global url
        url = dictWsdl[self.__config.getEnvironment()]
        self.__url = url

    """
		El metodo capture debe ser invocado siempre indicando el codigo del comercio que realizo la 
		transaccion. En el caso de comercios MALL, el codigo debe ser el codigo de la tienda virtual. 
		"""

    @staticmethod
    def capture(authorizationCode, captureAmount, buyOrder):

        client = WebpayCapture.get_client(url, config.getPrivateKey(), config.getPublicCert(), config.getWebPayCert())

        client.options.cache.clear()

        captureInput = client.factory.create('captureInput')

        captureInput.commerceId = config.getCommerceCode()
        captureInput.authorizationCode = authorizationCode
        captureInput.buyOrder = buyOrder
        captureInput.captureAmount = captureAmount

        try:
            captureOuput = client.service.capture(captureInput)
        except Exception as e:
            print str(e)

        return captureOuput

    @staticmethod
    def get_client(wsdl_url, our_keyfile_path, our_certfile_path, their_certfile_path):

        transport = HttpTransport()
        wsse = Security()

        return Client(
            wsdl_url,
            transport=transport,
            wsse=wsse,
            plugins=[
                WssePlugin(
                    keyfile=our_keyfile_path,
                    certfile=our_certfile_path,
                    their_certfile=their_certfile_path,
                ),
            ],
        )



from django.conf.urls import include, url
from django.conf.urls.i18n import i18n_patterns
from django.conf import settings
from django.contrib import admin
from django.contrib.staticfiles.urls import staticfiles_urlpatterns
from django.conf.urls.static import static

from oscar.app import application

admin.autodiscover()

urlpatterns = [
    url(r'^admin/', include(admin.site.urls)),
    url(r'^i18n/', include('django.conf.urls.i18n')),

    url(r'', include(application.urls)),
]

urlpatterns += [
    url(r'^webpay/', include('oscar_webpay.urls'))
]

if settings.DEBUG:
    import debug_toolbar

    urlpatterns += staticfiles_urlpatterns()
    urlpatterns += static(
        settings.MEDIA_URL, document_root=settings.MEDIA_ROOT)

    urlpatterns += [
        url(r'^__debug__/', include(debug_toolbar.urls)),
    ]





from django.utils.translation import ugettext_lazy as _


class CVVError(Exception):
    def __init__(self):
        self.message = _(u'CVV Error, please check and try again')
        super(CVVError, self).__init__()

class CARDError(Exception):
    def __init__(self):
        self.message = _(u'Card Error, please check and try again')

class OLDCARDError(Exception):
    def __init__(self):
        self.message = _(u'Your card has expired')

class ERR_TBK_CARD(Exception):
    def __init__(self):
        self.message = _(u'Wrong card type, please check and try again')

class ERR_TBK_NUMBER_QUOTAS(Exception):
    def __init__(self):
        self.message = _(u'Quota error, wrong number of cuotes, please check and try again')

class ERR_TBK_NUMBER_CARD(Exception):
    def __init__(self):
        self.message = _(u'Card Error, wrong number, please check and try again')

class ERR_TBK_CVV(Exception):
    def __init__(self):
        self.message = _(u'CVV Error, please check and try again')

class ERR_TIMEOUT(Exception):
    def __init__(self):
        self.message = _(u'Time limit exedeed, please check and try again')

class ERR_RUT(Exception):
    def __init__(self):
        self.message = _(u'Rut Error, please check and try again')

class ERR_CCV(Exception):
    def __init__(self):
        self.message = _(u'Security code error')

class ERR_EXPIRATION_DATE(Exception):
    def __init__(self):
        self.message = _(u'Error in the expiration date')

class ERR_AMOUNT(Exception):
    def __init__(self):
        self.message = _(u'Amount error')

class SOAPException(Exception):

    exceptions = {
        25: ERR_EXPIRATION_DATE(),
        33: ERR_AMOUNT(),
        52: CARDError(),
        86: OLDCARDError(),
        109: ERR_TBK_CARD(),
        111: ERR_TBK_NUMBER_QUOTAS(),
        112: ERR_TBK_NUMBER_CARD(),
        115: ERR_TBK_CVV(),
        272: ERR_TIMEOUT(),
        285: ERR_RUT()
    }

    def __init__(self, error_code):

        self.__generic_msg = \
            _("The possible causes of this rejection are:") \
            _("- Failure to enter the data of your Credit or Debit card (date and / or security code).")\
            _("- Your credit or debit card does not have enough balance.") \
            _("- Card not yet enabled in the financial system.")

        self.message = self.__generic_msg
        self.__error_code = error_code

    def __call__(self):
        if self.exceptions.get(self.__error_code, None) is not None:
            return self.exceptions[self.__error_code]
        return self



from django.conf.urls import patterns, url
from oscar_webpay import views


urlpatterns = patterns(
    '',
    url(r'^webpay_redirect/(?P<return_url_name>\w|\-)+/(?P<final_url_name>\w|\-)+/$', views.WebPayRedirectView.as_view(),
        name='webpay-redirect'),
    url(r'^webpay_form/$', views.WebPayRedirectForm.as_view(), name='webpay-form'),
    url(r'^webpay_success/$', views.WebPayPaymentSuccessView.as_view(), name='webpay-success'),
    url(r'^webpay_place_order/$', views.WebPayPaymentSuccessView.as_view(returning_from_webpay=False),
        name='webpay-place-order'),
    url(r'^webpay_thanks/$', views.WebPayThankYouView.as_view(), name='webpay-txns'),
    url(r'^webpay_fail/$', views.WebPayFail.as_view(), name='webpay-fail'),
    url(r'^webpay_cancel/$', views.WebPayCancel.as_view(), name='webpay-cancel'),
    url(r'^webpay_end_redirect/$', views.WebPayEndRedirect.as_view(), name='webpay-end-redirect'),
)



from oscar.apps.checkout import app

from apps.checkout import views


class CheckoutApplication(app.CheckoutApplication):
    payment_details_view = views.PaymentDetailsView


application = CheckoutApplication()



import os

# Django settings for oscar project.
PROJECT_DIR = os.path.dirname(__file__)
location = lambda x: os.path.join(os.path.dirname(os.path.realpath(__file__)), x)

DEBUG = True
TEMPLATE_DEBUG = True
SQL_DEBUG = True

ADMINS = (
    # ('Your Name', 'your_email@domain.com'),
)
EMAIL_BACKEND = 'django.core.mail.backends.console.EmailBackend'

USE_TZ = True

MANAGERS = ADMINS

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
    # Add 'postgresql_psycopg2', 'postgresql', 'mysql', 'sqlite3' or 'oracle'.
        'NAME': location('db.sqlite'),  # Or path to database file if using sqlite3.
        'USER': '',  # Not used with sqlite3.
        'PASSWORD': '',  # Not used with sqlite3.
        'HOST': '',  # Set to empty string for localhost. Not used with sqlite3.
        'PORT': '',  # Set to empty string for default. Not used with sqlite3.
    }
}
ATOMIC_REQUESTS = True

# Local time zone for this installation. Choices can be found here:
# http://en.wikipedia.org/wiki/List_of_tz_zones_by_name
# although not all choices may be available on all operating systems.
# On Unix systems, a value of None will cause Django to use the same
# timezone as the operating system.
# If running in a Windows environment this must be set to the same as your
# system time zone.
TIME_ZONE = 'Europe/London'

# Language code for this installation. All choices can be found here:
# http://www.i18nguy.com/unicode/language-identifiers.html
LANGUAGE_CODE = 'en-gb'

gettext_noop = lambda s: s
LANGUAGES = (
    ('en-gb', gettext_noop('British English')),
    ('zh-cn', gettext_noop('Simplified Chinese')),
    ('nl', gettext_noop('Dutch')),
    ('it', gettext_noop('Italian')),
    ('pl', gettext_noop('Polish')),
    ('ru', gettext_noop('Russian')),
    ('sk', gettext_noop('Slovak')),
    ('pt-br', gettext_noop('Brazilian Portuguese')),
    ('fr', gettext_noop('French')),
    ('de', gettext_noop('German')),
    ('ko', gettext_noop('Korean')),
    ('uk', gettext_noop('Ukrainian')),
    ('es', gettext_noop('Spanish')),
    ('da', gettext_noop('Danish')),
    ('ar', gettext_noop('Arabic')),
    ('ca', gettext_noop('Catalan')),
    ('cs', gettext_noop('Czech')),
    ('el', gettext_noop('Greek')),
)

SITE_ID = 1

# If you set this to False, Django will make some optimizations so as not
# to load the internationalization machinery.
USE_I18N = True

# If you set this to False, Django will not format dates, numbers and
# calendars according to the current locale
USE_L10N = True

# Absolute path to the directory that holds media.
# Example: "/home/media/media.lawrence.com/"
MEDIA_ROOT = location("public/media")

# URL that handles the media served from MEDIA_ROOT. Make sure to use a
# trailing slash if there is a path component (optional in other cases).
# Examples: "http://media.lawrence.com", "http://example.com/media/"
MEDIA_URL = '/media/'

# URL prefix for admin media -- CSS, JavaScript and images. Make sure to use a
# trailing slash.
# Examples: "http://foo.com/media/", "/media/".
# ADMIN_MEDIA_PREFIX = '/media/admin/'

STATIC_URL = '/static/'
STATIC_ROOT = location('public')

# Make this unique, and don't share it with anybody.
SECRET_KEY = '$)a7n&o80u!6y5t-+jrd3)3!%vh&shg$wqpjpxc!ar&p#!)n1a'

# List of callables that know how to import templates from various sources.
TEMPLATE_LOADERS = (
    'django.template.loaders.filesystem.Loader',
    'django.template.loaders.app_directories.Loader',
    #     'django.template.loaders.eggs.Loader',
)

TEMPLATE_CONTEXT_PROCESSORS = (
    "django.template.context_processors",
    "django.contrib.auth.context_processors.auth",
    "django.core.context_processors.request",
    "django.core.context_processors.debug",
    "django.core.context_processors.i18n",
    "django.core.context_processors.media",
    "django.core.context_processors.static",
    "django.contrib.messages.context_processors.messages",
    # Oscar specific
    'oscar.apps.search.context_processors.search_form',
    'oscar.apps.promotions.context_processors.promotions',
    'oscar.apps.checkout.context_processors.checkout',
    'oscar.core.context_processors.metadata',
)

MIDDLEWARE_CLASSES = (
    'django.middleware.locale.LocaleMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.contrib.flatpages.middleware.FlatpageFallbackMiddleware',
    'debug_toolbar.middleware.DebugToolbarMiddleware',
    'oscar.apps.basket.middleware.BasketMiddleware',
)

INTERNAL_IPS = ('127.0.0.1',)

ROOT_URLCONF = 'urls'

from oscar import OSCAR_MAIN_TEMPLATE_DIR

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [
            os.path.join(OSCAR_MAIN_TEMPLATE_DIR, 'templates'),
            OSCAR_MAIN_TEMPLATE_DIR,
            location('templates'),
        ],
        'APP_DIRS': True,
    }
]

# A sample logging configuration. The only tangible logging
# performed by this configuration is to send an email to
# the site admins on every HTTP 500 error.
# See http://docs.djangoproject.com/en/dev/topics/logging for
# more details on how to customize your logging configuration.
LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'formatters': {
        'verbose': {
            'format': '%(levelname)s %(asctime)s %(module)s %(process)d %(thread)d %(message)s'
        },
        'simple': {
            'format': '%(levelname)s %(message)s'
        },
    },
    'handlers': {
        'console': {
            'level': 'DEBUG',
            'class': 'logging.StreamHandler',
            'formatter': 'verbose'
        },
        'mail_admins': {
            'level': 'ERROR',
            'class': 'django.utils.log.AdminEmailHandler',
        },
    },
    'loggers': {
        'django.request': {
            'handlers': ['mail_admins'],
            'level': 'ERROR',
            'propagate': False,
        },
        'oscar.checkout': {
            'handlers': ['console'],
            'propagate': True,
            'level': 'INFO',
        },
    }
}

INSTALLED_APPS = [
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.sites',
    'django.contrib.messages',
    'django.contrib.admin',
    'django.contrib.flatpages',
    'django.contrib.staticfiles',
    # External apps
    # 1.5.7 still ships with South migrations in the wrong folder,
    # 1.5.8 contains a fix. Upgrade when released.
    # 'django_extensions',
    'debug_toolbar',
    # Apps from oscar
    'oscar_webpay',
    'compressor',
    'widget_tweaks',
]

from oscar import get_core_apps

INSTALLED_APPS = INSTALLED_APPS + get_core_apps([
    'apps.checkout'])

AUTHENTICATION_BACKENDS = (
    'oscar.apps.customer.auth_backends.Emailbackend',
    'django.contrib.auth.backends.ModelBackend',
)

LOGIN_REDIRECT_URL = '/accounts/'
APPEND_SLASH = True

# Oscar settings
from oscar.defaults import *

OSCAR_ALLOW_ANON_CHECKOUT = True

OSCAR_SHOP_TAGLINE = 'WebPay'

# Add WebPay dashboard stuff to settings
from django.utils.translation import ugettext_lazy as _

OSCAR_DASHBOARD_NAVIGATION.append(
    {
        'label': _('WebPay'),
        'icon': 'icon-globe',
        'children': [
            {
                'label': _('PayFlow transactions'),
                'url_name': 'paypal-payflow-list',
            },
            {
                'label': _('Express transactions'),
                'url_name': 'paypal-express-list',
            },
        ]
    })

# Haystack settings
HAYSTACK_CONNECTIONS = {
    'default': {
        'ENGINE': 'haystack.backends.simple_backend.SimpleEngine',
    },
}


# Put your own sandbox settings into an integration.py module (that is ignored
# by git).
try:
    from integration import *  # noqa
except ImportError:
    pass




"""
@author     Allware Ltda. (http://www.allware.cl)
@copyright  2016 Transbank S.A. (http://www.tranbank.cl)
@date       Jan 2015
@license    GNU LGPL
@version    2.0.1
"""
from django.conf import settings
from django.utils.translation import ugettext_lazy as _
from django.http import HttpResponse
import socket

from oscar_webpay.certificates import cert_normal

from suds.client import Client
from suds.wsse import Security, Timestamp
from wsse.suds import WssePlugin

from suds.transport.https import HttpTransport
import logging

from oscar_webpay.oscar_webpay_settings import OscarWebpaySettingError

logging.basicConfig()

config = None
url = None



class Dictionaries():
    """
	Configuracion de WSDL segun ambiente
	"""

    @staticmethod
    def dictionaryConfig():
        normal_config = getattr(settings, 'WEBPAY_NORMAL_MALL', None)
        config = normal_config.get('ENVIRONMENTS', None)
        if config is None:
            raise OscarWebpaySettingError(_('You must provide the ENVIRONMENTS variable for the settings'))

        return config


class WebpayMallNormal():
    def __init__(self, configuration):

        global config
        config = configuration
        self.__config = config

        dictWsdl = Dictionaries.dictionaryConfig()

        global url
        url = dictWsdl[self.__config.getEnvironment()]
        self.__url = url

    """
	initTransaction
	
	Permite inicializar una transaccion en Webpay. 
	Como respuesta a la invocacion se genera un token que representa en forma unica una transaccion.
	"""

    @staticmethod
    def initTransaction(amount, buyOrder, sessionId, urlReturn, urlFinal, stores):

        client = WebpayMallNormal.get_client(url, config.getPrivateKey(), config.getPublicCert(),
                                             config.getWebPayCert())
        client.options.cache.clear()

        init = client.factory.create('wsInitTransactionInput')

        init.wSTransactionType = client.factory.create('wsTransactionType').TR_MALL_WS

        init.commerceId = config.getCommerceCode()

        init.buyOrder = buyOrder
        init.sessionId = sessionId
        init.returnURL = urlReturn
        init.finalURL = urlFinal

        for data in stores:
            detail = client.factory.create('wsTransactionDetail')

            detail.commerceCode = data[0]
            detail.buyOrder = data[1]
            detail.amount = data[2]

            init.transactionDetails.append(detail)

        try:
            wsInitTransactionOutput = client.service.initTransaction(init)
        except Exception as e:
            str(e)

        return wsInitTransactionOutput

    """
	getTransaction
	
	Permite obtener el resultado de la transaccion una vez que 
	Webpay ha resuelto su autorizacion financiera.
	"""

    @staticmethod
    def getTransaction(token):

        client = WebpayMallNormal.get_client(url, config.getPrivateKey(), config.getPublicCert(),
                                             config.getWebPayCert())
        client.options.cache.clear()

        transactionResultOutput = client.service.getTransactionResult(token)
        acknowledge = WebpayMallNormal.acknowledgeTransaction(token)

        return transactionResultOutput

    """
	acknowledgeTransaction
	Indica  a Webpay que se ha recibido conforme el resultado de la transaccion
	"""

    @staticmethod
    def acknowledgeTransaction(token):

        client = WebpayMallNormal.get_client(url, config.getPrivateKey(), config.getPublicCert(),
                                             config.getWebPayCert())
        client.options.cache.clear()

        acknowledge = client.service.acknowledgeTransaction(token)

        return acknowledge

    @staticmethod
    def get_client(wsdl_url, our_keyfile_path, our_certfile_path, their_certfile_path):

        transport = HttpTransport()
        wsse = Security()

        return Client(
            wsdl_url,
            transport=transport,
            wsse=wsse,
            plugins=[
                WssePlugin(
                    keyfile=our_keyfile_path,
                    certfile=our_certfile_path,
                    their_certfile=their_certfile_path,
                ),
            ],
        )



"""
@author     Allware Ltda. (http://www.allware.cl)
@copyright  2016 Transbank S.A. (http://www.tranbank.cl)
@date       Jan 2015
@license    GNU LGPL
@version    2.0.1
"""

from django.http import HttpResponse
import socket

from oscar_webpay.certificates import cert_normal

from suds.client import Client
from suds.cache import NoCache
from suds.wsse import Security, Timestamp
from wsse.suds import WssePlugin

from suds.transport.https import HttpTransport
import logging

logging.basicConfig()

config = None
url = None;

from django.conf import settings
from django.utils.translation import ugettext_lazy as _
from oscar_webpay.oscar_webpay_settings import OscarWebpaySettingError


from logging import getLogger

logger = getLogger('oscar_webpay.transactions')


class Dictionaries():
    """
	Configuracion de WSDL segun ambiente
	"""

    @staticmethod
    def dictionaryConfig():
        normal_config = getattr(settings, 'WEBPAY_NORMAL', None)
        config = normal_config.get('ENVIRONMENTS', None)
        if config is None:
            raise OscarWebpaySettingError(_('You must provide the ENVIRONMENTS variable for the settings'))

        return config

class WebpayNormal():
    def __init__(self, configuration):
        global config;
        config = configuration;
        self.__config = config

        dictWsdl = Dictionaries.dictionaryConfig();

        global url;
        url = dictWsdl[self.__config.getEnvironment()];
        self.__url = url;

    @staticmethod
    def log_webpay_traffic(client, msg=""):
        logger.info("\n{}\n".format('='*100))
        logger.info(client.last_sent())
        logger.info("\n{}\n".format('='*100))
        logger.info(client.last_received())

    """
	initTransaction
	
	Permite inicializar una transaccion en Webpay. 
	Como respuesta a la invocacion se genera un token que representa en forma unica una transaccion.
	"""

    @staticmethod
    def initTransaction(amount, buyOrder, sessionId, urlReturn, urlFinal):
        client = WebpayNormal.get_client(url, config.getPrivateKey(), config.getPublicCert(), config.getWebPayCert())
        #client.options.cache.clear()
        init = client.factory.create('wsInitTransactionInput')

        init.wSTransactionType = client.factory.create('wsTransactionType').TR_NORMAL_WS

        init.commerceId = config.getCommerceCode();

        init.buyOrder = buyOrder;
        init.sessionId = sessionId;
        init.returnURL = urlReturn;
        init.finalURL = urlFinal;

        detail = client.factory.create('wsTransactionDetail');
        detail.amount = amount;

        detail.commerceCode = config.getCommerceCode();
        detail.buyOrder = buyOrder;

        init.transactionDetails.append(detail);
        init.wPMDetail = client.factory.create('wpmDetailInput');

        wsInitTransactionOutput = client.service.initTransaction(init);

        WebpayNormal.log_webpay_traffic(client)

        return wsInitTransactionOutput;



    """
	getTransaction
	
	Permite obtener el resultado de la transaccion una vez que 
	Webpay ha resuelto su autorizacion financiera.
	"""

    @staticmethod
    def getTransaction(token):
        client = WebpayNormal.get_client(url, config.getPrivateKey(), config.getPublicCert(), config.getWebPayCert());
        #client.options.cache.clear();
        transactionResultOutput = client.service.getTransactionResult(token);
        # acknowledge = WebpayNormal.acknowledgeTransaction(token);

        WebpayNormal.log_webpay_traffic(client)
        return transactionResultOutput;

    def acknowledgeTransaction(self, token):
        acknowledge = WebpayNormal.acknowledgeTransaction(token);

        return acknowledge

    """
	acknowledgeTransaction
	Indica  a Webpay que se ha recibido conforme el resultado de la transaccion
	"""

    @staticmethod
    def acknowledgeTransaction(token):
        client = WebpayNormal.get_client(url, config.getPrivateKey(), config.getPublicCert(), config.getWebPayCert());
        #client.options.cache.clear();

        acknowledge = client.service.acknowledgeTransaction(token);
        WebpayNormal.log_webpay_traffic(client)

        return acknowledge;

    @staticmethod
    def get_client(wsdl_url, our_keyfile_path, our_certfile_path, their_certfile_path):
        transport = HttpTransport()
        wsse = Security()

        return Client(
            wsdl_url,
            transport=transport,
            wsse=wsse,
            plugins=[
                WssePlugin(
                    keyfile=our_keyfile_path,
                    certfile=our_certfile_path,
                    their_certfile=their_certfile_path,
                ),
            ], cache=NoCache()
        )



"""
@author     Allware Ltda. (http://www.allware.cl)
@copyright  2016 Transbank S.A. (http://www.tranbank.cl)
@date       Jan 2015
@license    GNU LGPL
@version    2.0.1
"""
from django.conf import settings
from django.utils.translation import ugettext_lazy as _
from django.http import HttpResponse
import socket

from oscar_webpay.certificates import cert_normal

from suds.client import Client
from suds.wsse import Security, Timestamp
from wsse.suds import WssePlugin

from suds.transport.https import HttpTransport
import logging

from oscar_webpay.oscar_webpay_settings import OscarWebpaySettingError

logging.basicConfig()

config = None
url = None


class Dictionaries():
    """
	Configuracion de WSDL segun ambiente
	"""

    @staticmethod
    def dictionaryConfig():
        normal_config = getattr(settings, 'WEBPAY_NULLIFY', None)
        config = normal_config.get('ENVIRONMENTS', None)
        if config is None:
            raise OscarWebpaySettingError(_('You must provide the ENVIRONMENTS variable for the settings'))

        return config

class WebpayNullify():
    def __init__(self, configuration):

        global config
        config = configuration
        self.__config = config

        dictWsdl = Dictionaries.dictionaryConfig()

        global url
        url = dictWsdl[self.__config.getEnvironment()]
        self.__url = url

        """
        Permite solicitar a Webpay la anulacion de una transaccion
        realizada previamente y que se encuentra vigente.
        """

    @staticmethod
    def nullify(authorizationCode, authorizedAmount, buyOrder, nullifyAmount, commercecode):

        client = WebpayNullify.get_client(url, config.getPrivateKey(), config.getPublicCert(), config.getWebPayCert())

        client.options.cache.clear()

        nullificationInput = client.factory.create('nullificationInput')

        """ Codigo de autorizacion de la transaccion que se requiere anular """
        nullificationInput.authorizationCode = authorizationCode

        """ Monto autorizado de la transaccion que se requiere anular """
        nullificationInput.authorizedAmount = authorizedAmount

        """ Orden de Compra """
        nullificationInput.buyOrder = buyOrder

        """ Codigo de Comercio """
        if (commercecode == None):
            nullificationInput.commerceId = config.getCommerceCode()
        else:
            nullificationInput.commerceId = commercecode

        nullificationInput.nullifyAmount = nullifyAmount

        try:
            nullificationOutput = client.service.nullify(nullificationInput)
        except Exception as e:
            print str(e)

        return nullificationOutput

    @staticmethod
    def get_client(wsdl_url, our_keyfile_path, our_certfile_path, their_certfile_path):

        transport = HttpTransport()
        wsse = Security()

        return Client(
            wsdl_url,
            transport=transport,
            wsse=wsse,
            plugins=[
                WssePlugin(
                    keyfile=our_keyfile_path,
                    certfile=our_certfile_path,
                    their_certfile=their_certfile_path,
                ),
            ],
        )



from django.shortcuts import redirect
from oscar.apps.checkout import views

class PaymentDetailsView(views.PaymentDetailsView):
    """
    An example view that shows how to integrate WebPay methods.
    """

    # WebPay does not require the submission of the cardholder data
    # this info will be gathered in the WebPay Form.
    preview = True

    def get_context_data(self, **kwargs):
        ctx = super(PaymentDetailsView, self).get_context_data(**kwargs)
        return ctx

    def get(self, request, *args, **kwargs):
        return redirect('webpay-redirect')


from django.db import models
from django.utils.translation import ugettext_lazy as _

from oscar.core.loading import get_model


CoreTransaction = get_model('payment', 'Transaction')

class WebPayTransaction(CoreTransaction):

    TRANSACTION_TYPE_CHOICES = (
        ('VD', _(u'Debit sale')),
        ('VN', _(u'Normal sale')),
        ('VC', _(u'Quota sale')),
        ('SI', _(u'3 non-interest installments')),
        ('S2', _(u'2 non-interest installments')),
        ('NC', _(u'N non-interes installments'))
    )

    TRANSACTION_ERROR_CODE_CHOICES = (
        ('TSY', _(u"Successful transaction")),
        ('TSN', _(u"Failed transaction")),
        ('TO', _(u"Time limit exceeded")),
        ('ABO', _(u"Transaction aborted by card holder")),
        ('U3', _(u"Authentication internal error")),
    )

    token = models.CharField(_(u"Token"), max_length=255)
    buy_order = models.CharField(_(u"Buy order number"), max_length=26)
    commerce_code = models.CharField(_(u"Commerce code"), max_length=12)
    installments_amount = models.IntegerField(_(u"Installments amount"))
    currency = models.CharField(_(u"Currency"), max_length=3)
    auth_code = models.CharField(_(u"Authorization code"), max_length=6)
    card_number = models.CharField(_(u"Card number"), max_length=4)

    error_auth_code = models.IntegerField(_(u"Error code"), choices=TRANSACTION_ERROR_CODE_CHOICES)
    transaction_type = models.CharField(_(u"Transaction type"), max_length=255, choices=TRANSACTION_TYPE_CHOICES)

    def method(self):
        return _("WebPay: {}".format(self.transaction_type.display()))

